/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 *  WebSphere sMash
 *
 * (C) Copyright IBM Corp. 2007,2008  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */
dojo.provide("zero.assemble.form.resource.WebUIActivityRestStore");

dojo.declare("zero.assemble.form.resource.WebUIActivityRestStore", null, {
	
	//	summary:
	//		This data store is for webUI activity only. 
	// 	description:
	// 		where it behaves like this:
	// 		1. when performing fetchItemByIdentity on data store, item is returned 
	//			where the item is activity input data.
	//		2. when performing save on data store, it POSTs data to activity URL.			
	
	_REST_URL_SURFFIX : "?data",
	
	resourceUri : "",
	
	_changedItem : null,
	
	constructor: function(/* Object */ params) {
		this.resourceUri = params.resourceUri;
	},
	
	fetch: function(/* object */ keywordArgs) {
		// so far this implementation is not necessary but the form controller requires it.
		
		var scope = keywordArgs.scope || dojo.global;
		if(keywordArgs.onComplete){
			keywordArgs.onComplete.call(scope, []);
		}
	},
	
	getFeatures: function(){
		return { 
			"dojo.data.api.Read" : false, 
			"dojo.data.api.Identity" : true,
            "dojo.data.api.Notification": false,	
			"dojo.data.api.Write": true
		};
	},
	
	getResourceUri : function() {
		// summary:
		// 		ensure the resource url is legal.
		
		return ((this.resourceUri != '') ? this.resourceUri : location.pathname) + this._REST_URL_SURFFIX;
	},
	
	newItem: function(obj, parentInfo) { 
		//	summary:
		//		add 200 as valid status code.
		//		see zero.resource.RestStore.newItem()
		
		this._changedItem = obj;
        // this.onNew(obj,parentInfo);
		return obj;
	},
	
	save: function(/* object */ keywordArgs) {
		//	summary:
		//		save changed items.
		//	description:
		//		since xhr object will handle redirects transparently, 
		//		there's no way to make use of status 301/302/303 during 
		// 		redirect, it just returns the "Location" header.
		
		var scope = keywordArgs.scope || dojo.global;
		
		var xhrArgs = {
				url: this.resourceUri,
				handleAs: "json-comment-optional",
				postData: dojo.toJson(this._changedItem),
				preventCache: false,
				sync: false,
			    contentType: "application/json",
			    load: function(data, ioArgs) {
				 	this._changedItem = {};
				 	// json response contract of Assemble flow
			        var locationUrl = data.redirectURL;
			        if(keywordArgs.onComplete){
			        	// pass in the location so that the redirect location can be accessed
						keywordArgs.onComplete.call(scope, locationUrl);
					}
				},
				error: function(resp, ioArgs) {
			        var err = new Error(ioArgs.xhr.status + ": " + ioArgs.xhr.statusText);
			        err.responseText = ioArgs.xhr.responseText;
			        if(keywordArgs.onError){
						keywordArgs.onError.call(scope, err);
					}
				}
		};
		dojo.rawXhrPost(xhrArgs);
	},
	
	fetchItemByIdentity: function(keywordArgs) {
		var identity = keywordArgs.identity;
		// ignore identity based on webUI activity HTTP API
		var xhrArgs = {
  			url: this.getResourceUri(),
			handleAs: "json-comment-optional",
			preventCache: true
		};
		
		var scope =  keywordArgs.scope?keywordArgs.scope:dojo.global;
		var deferred = dojo.xhrGet(xhrArgs);
		deferred.addCallback(function(data) {
			var item = null;
			if (data !== null) {
				item = data;
			}
			if(keywordArgs.onItem){
				keywordArgs.onItem.call(scope, item);
			}
		});
		deferred.addErrback(function(error){
			if(keywordArgs.onError){
				keywordArgs.onError.call(scope, error);
			}
		});
	},
	
	setValue: function(/* item */ item, /* string */ attribute, /* almost anything */ value) {
		item[attribute] = value;
		this._changedItem = item;
	},
	
	deleteItem: function(item) {
		throw new Error("WebUIActivityRestStore doesn't support deletion.");
	}
});